<?php
/**
 * The template for displaying product content within loops.
 *
 * @package WooCommerce/Templates
 */

defined( 'ABSPATH' ) || exit;

global $product;

// Ensure visibility.
if ( empty( $product ) || ! $product->is_visible() ) {
    return;
}
?>
<div class="col">
    <div class="productDiv">
        <div class="productImg">
            <a href="<?php the_permalink(); ?>">
                <?php if ( has_post_thumbnail() ) {
                    the_post_thumbnail( 'woocommerce_thumbnail' );
                } else {
                    echo '<img src="' . esc_url( wc_placeholder_img_src() ) . '" alt="' . esc_attr__( 'Placeholder', 'woocommerce' ) . '" />';
                } ?>
            </a>
            <?php echo do_shortcode( '[yith_wcwl_add_to_wishlist]' ); ?>
            
        </div>

        <div class="productContent">
            <div class="productContentTop">
                <div class="productContentTopLeft">
                    <span class="productCat">
                        <?php
                        $categories = get_the_terms( get_the_ID(), 'product_cat' );
                        if ( $categories && ! is_wp_error( $categories ) ) {
                            echo esc_html( $categories[0]->name );
                        }
                        ?>
                    </span>
                    <a href="<?php the_permalink(); ?>"><h3 class="productName"><?php the_title(); ?></h3></a>
                </div>
                <div class="productRange">
                    <span class="star">
                        <img src="<?php echo bloginfo('template_url'); ?>/assets/images/star.svg" alt="star" />
                    </span>
                    <span><?php echo esc_html( $product->get_average_rating() ); ?>/5</span>
                </div>
            </div>

            <div class="productContentMiddle">
                <span class="price">
                    <?php echo $product->get_price_html(); ?>
                </span>
                <?php if ( $product->is_on_sale() ) : ?>
                    <span class="discount"><?php echo wc_format_sale_percentage( $product->get_regular_price(), $product->get_sale_price() ); ?>% off</span>
                <?php endif; ?>
            </div>

            <div class="productContentBottom">
        <a class="cmnbtn cmnbtn2 buy-now" 
   href="<?php echo esc_url( wc_get_checkout_url() . '?add-to-cart=' . $product->get_id() ); ?>">
   Buy Now
</a>


            <a class="cmnbtn cart add_to_cart_button ajax_add_to_cart" 
               href="<?php echo esc_url( $product->add_to_cart_url() ); ?>" 
               data-quantity="1" 
               data-product_id="<?php echo esc_attr( $product->get_id() ); ?>" 
               data-product_sku="<?php echo esc_attr( $product->get_sku() ); ?>" 
               aria-label="<?php echo esc_attr( $product->add_to_cart_text() ); ?>" 
               rel="nofollow">
               <i class="bi bi-cart"></i>
               <span class="loading-icon" style="display: none;">
                   <div class="spinner-border spinner-border-sm" role="status">
                       <span class="visually-hidden">Loading...</span>
                   </div>
               </span>
            </a>


            </div>
        </div>
    </div>
</div>
